import { Component, createEffect, createSignal, onCleanup } from 'solid-js';

import dispatch, { cancelDispatch } from '../../workers/dispatcher-main';
import VectorSource from 'ol/source/Vector';
import GeoJSON from 'ol/format/GeoJSON';
import {
  createCachedSignal,
  destroyCachedSignal,
} from '../../workers/cached-signals';

interface Props {
  rteId: string;
  vectorSource: VectorSource;
  context: any;
}

export const Rte: Component<Props> = ({ vectorSource, rteId, context }) => {
  const params = {
    id: rteId,
    method: 'getRte',
  };
  const rte = createCachedSignal(params);

  onCleanup(() => {
    console.log({
      caller: 'Rte / onCleanup',
      rteId,
      rte: rte(),
      params,
    });

    destroyCachedSignal(params);
  });

  createEffect(() => {
    console.log({ caller: 'Rte', vectorSource, rteId, rte: rte() });

    if (rte()) {
      let geo: any = {
        type: 'FeatureCollection',
        features: [
          {
            type: 'Feature',
            geometry: {
              type: 'Point',
              coordinates: [rte().rtept[0].$.lon, rte().rtept[0].$.lat],
            },
            properties: {
              type: 'rte-start',
              id: rteId,
              context: { ...context, rte, rteId },
            },
            id: `${rteId}/start`,
          },
          {
            type: 'Feature',
            geometry: {
              type: 'LineString',
              coordinates: rte().rtept.map((rtept: any) => [
                rtept.$.lon,
                rtept.$.lat,
              ]),
            },
            properties: {
              type: 'rte',
              id: rteId,
              context: { ...context, rte, rteId },
            },
            id: rteId,
          },
          {
            type: 'Feature',
            geometry: {
              type: 'Point',
              coordinates: [rte().rtept.at(-1).$.lon, rte().rtept.at(-1).$.lat],
            },
            properties: {
              type: 'rte-finish',
              id: rteId,
              context: { ...context, rte, rteId },
            },
            id: `${rteId}/end`,
          },
        ],
      };

      const features = new GeoJSON().readFeatures(geo);
      console.log({ caller: 'Rte', features });
      [`${rteId}/start`, rteId, `${rteId}/end`].forEach((id) => {
        const feature = vectorSource.getFeatureById(id);
        if (feature) {
          vectorSource.removeFeature(feature);
        }
      });
      vectorSource.addFeatures(features);
    }
  });

  return <></>;
};

export default Rte;
